#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:08:34 2018

@author: andrew
"""

#%%
#gives statistics of all iamges in a directory
def stats(location):
    images = glob.glob(location + "/*.fits")
    means = []
    stdev = []
    for i in images:
        hdu = fits.open(i)
        data = hdu[0].data
        means.append(np.mean(data))
        stdev.append(np.std(data))
        hdu.close()
    mean = np.mean(means)
    std = np.mean(stdev)
    print "Average image background value = %d | Average standard deviation = %d" % (mean, std)

#%%
#get statistics and plots for a single pixel value across many images
def pixel_stats(location, i, j):
    import matplotlib.pyplot as plt
    x = 0
    images = glob.glob(location + '/data/*.fits')
    pixel = []
    for im in images:
        hdu = fits.open(im)
        data = hdu[0].data
        site = hdu[0].header['SITE']
        pixel.append(data[i, j])
        hdu.close()
        if data[i, j] < 300:
            os.system("rm %s" % (im))
            x += 1
    samples = len(pixel)
    Max = np.max(pixel)
    Min = np.min(pixel)
    mean = np.mean(pixel)
    median = np.median(pixel)
    stdev = np.std(pixel)
    threesig_up = mean + 3*stdev
    threesig_down = mean - 3*stdev
    twosig_up = mean + 2*stdev
    twosig_down = mean - 2*stdev   
    plt.hist(pixel, bins=15, rwidth=0.7)
    plt.title("Pixel Values (%d, %d) [%d samples]" % (i, j, samples))
    if Max < threesig_up and Min > threesig_down:
        if Max < twosig_up and Min > twosig_down:
            print "all pixels contained within 2 sigma"
        else:
            print "all pixels contained within 3 sigma"
    else:
        print "pixels exceed 3 sigma"
    print "max = %d\nmin = %d\nmean = %d\nmedian = %d\nstdev = %d" % (Max, Min, mean, median, stdev)
    print x
    
#%%
#get statistics and plots for a group of images
def image_stats(location):
    x = 0
    images = glob.glob(location + '/*.fits')
    sums = []
    means = []
    stdev = []
    Max = []
    Min = []
    median = []
    for im in images:
        hdu = fits.open(im)
        array = hdu[0].data
        array = array[387:550, 700:850]
#        molnum = hdu[0].header['MOLNUM']
#        siteid = hdu[0].header['SITEID']
        sums.append(np.sum(array))
        stdev.append(np.std(array))
        Max.append(np.max(array))
        Min.append(np.min(array))
        median.append(np.median(array))
#        print np.mean(array)
#        x += 1
#        means.append(np.mean(array))
        if np.mean(array) < 375 and np.mean(array) > 330:
            x += 1
            print im
            means.append(np.mean(array))
            os.system("cp %s /home/andrew/images2/data" % (im))
#        threesig_up = np.mean(array) + 3*np.std(array)
#        threesig_down = np.mean(array) - 3*np.std(array)
#        twosig_up = np.mean(array) + 2*np.std(array)
#        twosig_down = np.mean(array) - 2*np.std(array)
#        if np.max(array) < threesig_up and np.min(array) > threesig_down:
#            if Max < twosig_up and Min > twosig_down:
#                print "all pixels contained within 2 sigma"
#            else:
#                print "all pixels contained within 3 sigma"
#        else:
#            print "pixels exceed 3 sigma"
#        hdu.close()
#    samples = len(images)
#    plt.hist(sums, bins = 20, rwidth = 0.7)
#    plt.title("Sum Image Values [%d samples]" % (samples))
#    plt.figure()
    plt.hist(means, bins = 100, rwidth = 0.7)
#    plt.title("Mean Residual Values (ADU) [%d M31 Off-Core frames]" % (samples))
    plt.figure()
#    plt.hist(stdev, bins = 20, rwidth = 0.7)
#    plt.title("Standard Deviations (ADU) [%d M31 Off-Core frames]" % (samples))
#    plt.figure()
#    print "mean = %d" % (float(np.mean(means)))
    #print "max = %d\nmin = %d\nmean = %d\nmedian = %d\nstdev = %d" % (Max, Min, mean, median, stdev)
#    for i in range(len(sums)):
#        sums[i] *= 10**-6
#    plt.figure()
#    plt.imshow(array, cmap='gray')
#    plt.figure()
    print x
    print images[np.argmin(means)], images[np.argmax(means)]
    return np.std(means), np.mean(means), np.median(sums), array
    


import align_astroalign
import align_template
from ref_image import ref_image
from initialize import loc
import subtract_hotpants
import subtract_ibi
import subtract_numpy
import combine_swarp
import combine_numpy
import sex
import check_saturation

if __name__ == '__main__':
    location = input("Enter path to data directory: ")
    sat = check_saturation.check_saturate(location)
    if sat == 0:
        ref_image(location)
        alignment = input("Enter alignment method (enter iraf or astroalign): ")
        if alignment == "iraf":
            import align_iraf
            align_iraf.align(location)
        elif alignment == "astroalign":
            align_astroalign.align2(location)
        else:
            print("Error: unknown method")
    else:
        check = input("Saturated images found, continue image alignment? (y/n): ")
        if check == 'y':
            move = input("Move saturated images to SDI archives before continuing? (y/n): ")
            if move == 'y':
                check_saturation.move_arch(sat)
                ref_image(location)
                alignment = input("Enter alignment method (enter iraf or astroalign): ")
                if alignment == "iraf":
                    import align_iraf
                    align_iraf.align(location)
                elif alignment == "astroalign":
                    align_astroalign.align2(location)
                else:
                    print("Error: unknown method")
            else:
                ref_image(location)
                alignment = input("Enter alignment method (enter iraf or astroalign): ")
                if alignment == "iraf":
                    import align_iraf
                    align_iraf.align(location)
                elif alignment == "astroalign":
                    align_astroalign.align2(location)
                else:
                    print("Error: unknown method")
    method = input("Choose combination method-- numpy (default), swarp, or iraf: ")
    if method == "swarp":
        combine_swarp.swarp(location)
    elif method == "numpy" or method == "":
        combine_numpy.combine_median(location)
    elif method == "iraf":
        import combine_iraf
        combine_iraf.combine(location)
    else:
        print("Error: unknown method entered")
    path = location[:-5]
    method = input("Choose subtraction method: numpy (default), hotpants, image-by-image, or iraf: ")
    if method == 'numpy' or method == '':
        subtract_numpy.subtract2(path)
    elif method == 'hotpants':
        subtract_hotpants.hotpants(path)
    elif method == 'image-by-image':
        subtract_ibi.subtract3(path)
    elif method == 'iraf':
        import subtract_iraf
        subtract_iraf.subtract(path)
    else:
        print("Error: Unknown method")
    ask = input("Run sextractor on residual images? (y/n): ")
    if ask == 'y':
        sex.sex(path)
    elif ask != 'y' and ask != 'n':
        print("Error: unknown input")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Jun 12 10:28:32 2018

@author: andrew
"""

import glob
import numpy as np
from astropy.io import fits
import os
from initialize import loc

#%%
#checks all fits images in a directory for saturation
def check_saturate(location):
    print("checking images for saturation...")
    Max = []
    im = []
    m = []
    x = 0
    y = 0
    z = 0
    images = glob.glob(location + "/*.fits")
    length = len(location) + 6
    for i in images:
        hdu = fits.open(i)
        lin = hdu[0].header['SATURATE']
        data = hdu[0].data
#        rows = np.size(data, axis=0)
#        cols = np.size(data, axis=1)
#        for r in np.arange(rows):
#            for c in np.arange(cols):
#                if data[r, c] > lin:
#                    x += 1
        sat = ((data>lin)).sum()
#        ind = np.unravel_index(np.argmax(data, axis=None), data.shape)
#        excess = data[ind[0], ind[1]] - lin
        if sat > 3:
#            print "\n%s saturated | # saturated pixels = %d | max pixel location = (%d, %d)\nmax value over linearity limit = %d" % (i[length:], x, ind[0], ind[1], excess)
            y += 1
            im.append(i)
            m.append(np.max(data))
        Max.append(np.max(data))
#        x = 0
        sat = 0
        hdu.close()
    if y > 0:
        print("\n%d/%d saturated images" % (y, len(images)))
        print("\naverage saturation level (ADU) = %d" % (np.mean(m)-lin))
        return im
    if y == 0:
        diff = lin - np.max(Max)
        print("\nno saturated images in %s" % (location))
        print("\nclosest value to saturation = %d" % (np.max(Max)))
        print("\ndifference between this value and saturation level = %d" % (diff))
        return y
    
#%%
#move images into archives
def move_arch(images):
    archive_data_loc = loc + "/sdi/archive/data"
    for i in images:
        os.system("mv %s %s" % (i, archive_data_loc))
    print("Saturated images moved to SDI archives")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 12:59:46 2018

@author: andrew
"""

import glob
import astroalign
from astropy.io import fits
import numpy as np

#%%
#align aligned images to template using astroalign package
def align3(location):
    x = 1
    images = glob.glob(location + "/data/*_A_.fits")
    temp = glob.glob(location + "/templates/*.fits")
    hdu2 = fits.open(temp[0])
    data2 = hdu2[0].data
    data2 = np.array(data2, dtype="float64")  
    for i in images:
        hdu1 = fits.open(i)
        data1 = hdu1[0].data
        data1 = np.array(data1, dtype="float64")
        aligned = astroalign.register(data1, data2)
        aligned_name = i[:-8] + "_AT_.fits"
        hdu = fits.PrimaryHDU(aligned, header=hdu1[0].header)
        hdu.writeto(aligned_name)
        hdu1.close()
        percent = float(x)/float(len(images)) * 100
        print "%.3f%% aligned..." % (percent)
        x += 1
    hdu2.close()


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:04:11 2018

@author: andrew
"""

import glob
import os

#%%
#subtract using hotpants
def hotpants(location):
    x = 0
    images = glob.glob(location + "/data/*_A_.fits")
    template = glob.glob(location + "/templates/*.fits")
    outputs = []
    length = len(location) + 6
    if len(template) == 1:
        for i in images:
            outputs.append(location + "/residuals/" + i[length:-8] + "_hotpants.fits")
            os.system("./hotpants -inim %s -tmplim %s -outim %s" % (images[x], template[0], outputs[x]))
            x += 1
            per = float(x)/float(len(images)) * 100
            print("%d subtracted..." % (per))
    else:
        print("error with number of templates")


from initialize import loc
import subtract_hotpants
import subtract_ibi
import subtract_numpy

if __name__ == '__main__':
    path = input("Enter path to exposure time directory: ")
    method = input("Choose subtraction method: numpy (default), hotpants, image-by-image, or iraf: ")
    if method == 'numpy' or method == '':
        subtract_numpy.subtract2(path)
    elif method == 'hotpants':
        subtract_hotpants.hotpants(path)
    elif method == 'image-by-image':
        subtract_ibi.subtract3(path)
    elif method == 'iraf':
        import subtract_iraf
        subtract_iraf.subtract(path)
    else:
        print("Error: Unknown method")


import combine_swarp
import combine_numpy

if __name__ == '__main__':
    location = input("Enter path to data directory: ")
    method = input("Choose combination method-- numpy (default), swarp, or iraf: ")
    if method == "swarp":
        combine_swarp.swarp(location)
    elif method == "numpy" or method == "":
        combine_numpy.combine_median(location)
    elif method == "iraf":
        import combine_iraf
        combine_iraf.combine(location)
    else:
        print("Error: unknown method entered")



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 12:58:40 2018

@author: andrew
"""

import glob
import os
from pyraf import iraf
from initialize import loc

#%%
#makes or updates list of all fits images in a directory
def inputlist(location):
    x = 0
    file = open("%s/inputs.txt" % (location), "w+")
    images = glob.glob(location + "/*_N_.fits")
    ref = glob.glob(location + "/*_ref_A_.fits")
    if len(ref) == 1:
        file.write(str(ref[0]) + "\n")
        x += 1
    else:
        print("problem with reference image")
    for i in images:
        file.write(i + "\n")
        x += 1
    file.close()
    print("input list updated for %s (%d images)" % (location, x))
    
#%%
#makes or updates list of all ouput names of fits files in a directory
def outputlist(location):
    x = 0
    file = open("%s/outputs.txt" % (location), "w+")
    images = glob.glob(location + "/*_N_.fits")
    ref = glob.glob(location + "/*_ref_A_.fits")
    if len(ref) == 1:
        file.write(str(ref[0]) + "\n")
        x += 1
    else:
        print("problem with reference image")
    for i in images:
        i = i[:-8]
        file.write(i + "_A_.fits" + "\n")
        x += 1
    file.close()
    print("output list updated for %s (%d images)" % (location, x))
    
#%%
#aligns images in a directory using IRAF imalign
def align(location):
    inputlist(location)
    outputlist(location)
    ref_image = glob.glob(location + "/*_ref_A_.fits")
    input_list = glob.glob(location + "/inputs*")
    output_list = glob.glob(location + "/outputs*")
    coordinates = glob.glob(location + "/*.coo")
    if input_list == []:
        print("no input list found")
    elif output_list == []:
        print("no output list found")
    elif coordinates == []:
        print("no coordinate list found")
    elif ref_image == []:
        print("no reference image found")
    elif len(input_list) > 1:
        print("too many input lists")
    elif len(output_list) > 1:
        print("too many ouptut lists")
    elif len(coordinates) > 1:
        print("too many coordinate lists")
    elif len(ref_image) > 1:
        print("too many reference images")
    else:
        try:
            ref_image = ref_image[0]
            input_list = "@" + input_list[0]
            output_list = "@" + output_list[0]
            coordinates = coordinates[0]
            iraf.imalign(input=input_list, reference=ref_image, coords=coordinates, output=output_list, boxsize = 20, bigbox = 30)
            images = glob.glob(location + "/*_N_.fits")
            for i in images:
                os.system("mv %s %s/sdi/archive/data" % (i, loc))
            print("alignment successful!")
            print("moved non-aligned _N_ images to sdi archive")
        except iraf.IrafError:
            print("alignment failed, check parameters")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:00:21 2018

@author: andrew
"""

from time import strftime
from time import gmtime
import glob
from initialize import loc
from pyraf import iraf
import os

#%%
#combine all aligned images in a directory to make a template and move template to template directory
def combine(location, method = "median"):
    location = location[:-5]
    time = strftime("%Y-%m-%d %H:%M:%S", gmtime())
    images = glob.glob(location + "/data" + "/*_A_.fits")
    og_templates = glob.glob(location + "/templates/*.fits")
    ref = glob.glob(location + "/data" + "/*_ref_.fits")
    log_loc = location + "/templates/log.txt"
    tlist_loc = location + "/templates/template_inputs.txt"
    log_list = open(log_loc, "a+")
    template_list = open(tlist_loc, "w+")
    for i in images:
        template_list.write(str(i) + "\n")
    template_list.close()
    if images == []:
        print("no aligned images to combine\n")
    else:
        output_image = log_loc[:-7] + str(len(images)) + "_" + method + ".fits"
        try:
            print("images being combined...\n")
            iraf.imcombine(input="@" + tlist_loc, output=output_image, combine=method)
            log_list.write("template updated at %s UTC | method = %s (IRAF) | images = %d\n" % (str(time), method, len(images)))
            log_list.close()
            if len(og_templates) > 0:
                for o in og_templates:
                    os.system("mv %s %s/sdi/archive/templates" % (o, loc))
            print("image combination successful!\ntemplate log updated\n")
        except:
            print("image combination failed\n")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 12:58:10 2018

@author: andrew
"""

import glob
import numpy as np
from astropy.io import fits
import os

#%%
#rename first image in directory to be the reference image
def ref_image(location):
    noise = []
    ref = glob.glob(location + "/*_ref_A_.fits")
    length = len(location) + 1
    if ref == []:
        images = glob.glob(location + "/*.fits")
        for i in images:
            hdu = fits.open(i)
            data = hdu[0].data
            noise.append(np.mean(data))
        im = images[np.argmin(noise)]
        reference = location + "/" + im[length:-8] + "_ref_A_.fits"
        os.system("mv %s %s" % (im, reference))
        print("designated %s as the reference image in this directory\n" % (im))
    else:
        print("reference image already exists in this directory\n")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:01:41 2018

@author: andrew
"""

import glob
from pyraf import iraf

#%%
#subtracts an image or group of images from a specified template image
def subtract(location):
    tlist_loc = location + "/templates/template_inputs.txt"
    template_loc = glob.glob(location + "/templates/*.fits")
    res_outputs = open(location + "/residuals/residual_outputs.txt", "w+")
    res_outputs_loc = location + "/residuals/residual_outputs.txt"
    images = glob.glob(location + "/data" + "/*_A_.fits")
    length = len(location) + 6
    for i in images:
        res_outputs.write(location + "/residuals/" + i[length:-5] + "residual.fits\n")
    res_outputs.close()
    if len(template_loc) == 1:
        try:
            iraf.imarith(operand1="@" + str(tlist_loc), op="-", operand2=str(template_loc[0]), result="@" + str(res_outputs_loc))
            print("image subtraction successful")
        except:
            print("subtraction failed")
    else:
        print("problem with template image")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:04:32 2018

@author: andrew
"""

import os
import glob

#%%
#runs SExtractor on all residual images in a directory
def sex(location):
    x = 0
    sources = location + "/sources"
    residuals = location + "/residuals"
    check = os.path.exists(sources)
    length = len(residuals) + 1
    if check == False:
        os.system("mkdir %s" % (sources))
    images = glob.glob(residuals + "/*.fits")
    for i in images:
        name = i[length:-5]
        os.system("sextractor %s > %s/%s.txt" % (i, sources, name))
        x += 1
        per = float(x)/float(len(images)) * 100
        print("%d sextracted..." % (per))
    print("SExtracted %d images, catalogues places in 'sources' directory\n" % (len(images)))



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 12:54:19 2018

@author: andrew
"""

import os
import zipfile
from astropy.io import fits
import glob
from initialize import loc
from initialize import create

#%%
#move downloaded LCO data from download_loc to temp folder
def move(download_loc):
    temp = "%s/sdi/temp" % (loc)
    num = 1
    x = []
    y = []
    for root, dirs, files in os.walk(download_loc):
        for f in files:
            if f[:3] == "lco":
                os.system('mv "%s/%s" %s' % (download_loc, f, temp))
                x.append(num)
        for d in dirs:
            if d[:3] == "lco":
                os.system('mv "%s/%s" %s' % (download_loc, d, temp))
                y.append(num)
    print("moved %d files and %d directories from %s to /sdi/temp\n" % (sum(x), sum(y), download_loc))
    
#%%
#unzips and funpacks downloaded LCO data in temp directory
def process():
    num = 1
    x = []
    y = []
    d = 0
    temp = "%s/sdi/temp" % (loc)
    zipfiles = glob.glob("%s/*.zip" % (temp))
    for i in zipfiles:
        zip_ref = zipfile.ZipFile(i, 'r')
        zip_ref.extractall(temp)
        zip_ref.close()
        os.remove(i)
        x.append(num)
        d =+ 1
    for d in os.listdir(temp):
        if os.path.isdir(temp+"/"+d)==True:
            for j in os.listdir(temp+"/"+d):
                if j.endswith(".fz"):
                    os.system('funpack "%s/%s/%s"' % (temp, d, j))
                    os.system('rm "%s/%s/%s"' % (temp, d, j))
                    y.append(num)
    print("%d files unzipped and %d images funpacked\n" % (sum(x), sum(y)))
    
#%%
#moves data from /sdi/temp to its respective target directory
def movetar(tar):
    check = os.path.exists("%s/sdi/targets/%s" % (loc, tar))
    if check == False:
        os.system("mkdir %s/sdi/targets/%s" % (loc, tar))
        os.system("mkdir %s/sdi/targets/%s/raw_data" % (loc, tar))
    one = 1
    x = []
    y = []
    temp = "%s/sdi/temp" % (loc)
    data = "%s/sdi/targets/%s" % (loc, tar)
    raw_data = "%s/sdi/targets/%s/raw_data" % (loc, tar)
    for d in os.listdir(temp):
        files = glob.glob(temp + "/" + d + "/*.fits")
        for f in files:
            if f[-7] == "9":
                os.system("mv %s %s" % (f, data))
                x.append(one)
            if f[-7] == "0" or f[-7] == "1":
                os.system("mv '%s' %s" % (f, raw_data))
                y.append(one)
        try:
            if os.listdir(temp + "/" + d) == []:
                os.rmdir(temp + "/" + d)
                print("removed %s because it became empty\n" % (d))
        except NotADirectoryError:
            pass
    print("moved %d images to %s\n" % (sum(x), data))
    print("moved %d raw images to %s\n" % (sum(y), raw_data))
    if check == False:
        print("created %s directory in %s/sdi/targets\n" % (tar, loc))
        
#%%
#group images of same RA and DEC together in their own directories
def rename(tar):
    x = []
    one = 1
    target = "%s/sdi/targets/%s" % (loc, tar)
    length = len(target) + 1
    for f in glob.glob("%s/*.fits" % (target)):
        F = f[length:]
        hdu = fits.open(f)
        ra = hdu[0].header['CAT-RA']
        dec = hdu[0].header['CAT-DEC']
        fltr = hdu[0].header['FILTER1']
        exp = round(int(hdu[0].header['EXPTIME']))
        stoptime = hdu[0].header['UTSTOP']
        check = os.path.exists("%s/%s_%s" % (target, ra, dec))
        if check == False:
            os.system("mkdir %s/%s_%s" % (target, ra, dec))
            os.system("mkdir %s/%s_%s/%s" % (target, ra, dec, fltr))
            os.system("mkdir %s/%s_%s/%s/%s" % (target, ra, dec, fltr, exp))
            create("%s/%s_%s/%s/%s" % (target, ra, dec, fltr, exp))
            os.system("mv %s %s/%s_%s/%s/%s/data" % (f, target, ra, dec, fltr, exp))
            os.system("mv %s/%s_%s/%s/%s/data/%s %s/%s_%s/%s/%s/data/%s_N_.fits" % (target, ra, dec, fltr, exp, F, target, ra, dec, fltr, exp, stoptime))
        if check == True:
            check2 = os.path.exists("%s/%s_%s/%s" % (target, ra, dec, fltr))
            if check2 == False:
                os.system("mkdir %s/%s_%s/%s" % (target, ra, dec, fltr))
                os.system("mkdir %s/%s_%s/%s/%s" % (target, ra, dec, fltr, exp))
                create("%s/%s_%s/%s/%s" % (target, ra, dec, fltr, exp))
                os.system("mv %s %s/%s_%s/%s/%s/data" % (f, target, ra, dec, fltr, exp))
                os.system("mv %s/%s_%s/%s/%s/data/%s %s/%s_%s/%s/%s/data/%s_N_.fits" % (target, ra, dec, fltr, exp, F, target, ra, dec, fltr, exp, stoptime))
            if check2 == True:
                check3 = os.path.exists("%s/%s_%s/%s/%s" % (target, ra, dec, fltr, exp))
                if check3 == False:
                    os.system("mkdir %s/%s_%s/%s/%s" % (target, ra, dec, fltr, exp))
                    create("%s/%s_%s/%s/%s" % (target, ra, dec, fltr, exp))
                    os.system("mv %s %s/%s_%s/%s/%s/data" % (f, target, ra, dec, fltr, exp))
                    os.system("mv %s/%s_%s/%s/%s/data/%s %s/%s_%s/%s/%s/data/%s_N_.fits" % (target, ra, dec, fltr, exp, F, target, ra, dec, fltr, exp, stoptime))
                if check3 == True:
                    os.system("mv %s %s/%s_%s/%s/%s/data" % (f, target, ra, dec, fltr, exp))
                    os.system("mv %s/%s_%s/%s/%s/data/%s %s/%s_%s/%s/%s/data/%s_N_.fits" % (target, ra, dec, fltr, exp, F, target, ra, dec, fltr, exp, stoptime))
        x.append(one)
        hdu.close()
    print("%d images grouped into location directories\n" % (sum(x)))
    


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:01:19 2018

@author: andrew
"""

import glob
from initialize import loc
import os
from time import strftime
from time import gmtime

#%%
#median combine images using SWarp
def swarp(location):
    location = location[:-5]
    images = glob.glob(location + "/data/*_A_.fits")
    config_loc = "/home/andrew/sdi/pipeline/scripts/default.swarp"
    template = location + "/templates/swarp_median_" + str(len(images)) + ".fits"
    with open(config_loc, 'r') as config:
        data = config.readlines()
        config.close()
    data[4] = "IMAGEOUT_NAME" + "        " + template + "\n"
    with open(config_loc, 'w') as config:
        config.writelines(data)
        config.close()
    time = strftime("%Y-%m-%d %H:%M:%S", gmtime())
    og_templates = glob.glob(location + "/templates/*.fits")
    log_loc = location + "/templates/log.txt"
    tlist_loc = location + "/templates/template_inputs.txt"
    log_list = open(log_loc, "a+")
    template_list = open(tlist_loc, "w+")
    for i in images:
        template_list.write(str(i) + "[0]" + "\n")
    template_list.close()
    if images == []:
        print("no aligned images to combine\n")
    else:
        try:
            print("images being combined...\n")
            os.system("swarp @%s -c %s" % (tlist_loc, config_loc))
            log_list.write("template updated at %s UTC | method = median (SWarp) | images = %d\n" % (str(time), len(images)))
            log_list.close()
            if len(og_templates) > 0:
                for o in og_templates:
                    os.system("mv %s %s/sdi/archive/templates" % (o, loc))
            print("image combination successful!\ntemplate log updated\n")
        except:
            print("image combination failed\n")


import obtain
from initialize import loc
from initialize import create

if __name__ == '__main__':
    download_location = input("Enter LCO data location (leave blank for default=%s/Downloads): " % (loc))
    if download_location == "":
        download_location = "%s/Downloads" % (loc)
    obtain.move(download_location)
    obtain.process()
    check = input("Move data into target directory? (y/n): ")
    if check == "y":
        tar = input("Enter target: ")
        obtain.movetar(tar)
        obtain.rename(tar)
    elif check != "y" and check != "n":
        print("Error: unknown Input")


import align_astroalign
import align_template
from ref_image import ref_image
import check_saturation

if __name__ == '__main__':
    location = input("Enter path to data directory: ")
    sat = check_saturation.check_saturate(location)
    if sat == 0:
        ref_image(location)
        alignment = input("Enter alignment method (enter iraf or astroalign): ")
        if alignment == "iraf":
            import align_iraf
            align_iraf.align(location)
        elif alignment == "astroalign":
            align_astroalign.align2(location)
        else:
            print("Error: unknown method")
    else:
        check = input("Saturated images found, continue image alignment? (y/n): ")
        if check == 'y':
            move = input("Move saturated images to SDI archives before continuing? (y/n): ")
            if move == 'y':
                check_saturation.move_arch(sat)
                ref_image(location)
                alignment = input("Enter alignment method (enter iraf or astroalign): ")
                if alignment == "iraf":
                    import align_iraf
                    align_iraf.align(location)
                elif alignment == "astroalign":
                    align_astroalign.align2(location)
                else:
                    print("Error: unknown method")
            else:
                ref_image(location)
                alignment = input("Enter alignment method (enter iraf or astroalign): ")
                if alignment == "iraf":
                    import align_iraf
                    align_iraf.align(location)
                elif alignment == "astroalign":
                    align_astroalign.align2(location)
                else:
                    print("Error: unknown method")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:02:21 2018

@author: andrew
"""

import glob
import copy
from astropy.io import fits
import numpy as np

#%%
def subtract3(location):
    y = 0
    images = glob.glob(location + "/data/*_A_.fits")
    images_copy = copy.deepcopy(images)
    length = len(location) + 6
    for i in images:
        hdu1 = fits.open(i)
        data1 = hdu1[0].data
        data1 = np.array(data1, dtype="float64")
        images_copy.remove(i)
        y += 1
        for j in images_copy:
            hdu2 = fits.open(j)
            data2 = hdu2[0].data
            data2 = np.array(data2, dtype="float64")
            sub = data1 - data2
            sub_name = location + "/residuals/" + j[length:-5] + "residual%d.fits" % (y)
            hdu = fits.PrimaryHDU(sub, header=hdu2[0].header)
            hdu.writeto(sub_name)
            hdu1.close()
        hdu1.close()
    print("image subtraction successful")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed May  2 15:45:30 2018

@author: andrew
"""
import random
import os
import glob
from astropy.io import fits
import numpy as np
import matplotlib.pyplot as plt
import shutil
from initialize import loc

#%%
#make a copy of target directory in the simulations directory
def copy_to_sim(tar):
    sim_path = loc + "/sdi/simulations"
    tar_dir_path = loc + "/sdi/targets"
    try:
        shutil.copytree(tar_dir_path + "/" + tar, sim_path + "/" + tar)
        print("Copy target directory created in simulations directory")
    except Exception as e:
        print(e)
        
#%%
#create fakes directory for a given target
def fake_dir(location):
    fake_dir = location + "/fakes"
    if os.path.exists(fake_dir) == False:
        os.mkdir(fake_dir)
    else:
        print("Fakes directory already exists for this target")

#%%
def fakes(location, mag):
    fakes = location + "/fakes"
#    clear contents of directory
    for i in os.listdir(fakes):
        path = os.path.join(fakes, i)
        try:
            if os.path.isfile(path):
                os.unlink(path)
        except Exception as e:
            print(e)
    FAKE = glob.glob(location +"/data/*FAKE.fits")
    if FAKE != []:
        for i in FAKE:
            os.system("rm %s" % (i))
    fake_res = glob.glob(location + "/residuals/*FAKE_residual.fits")
    if fake_res != []:
        for i in fake_res:
            os.system("rm %s" % (i))
    catalogs = glob.glob(location + "/sources/*.cat")
    if catalogs != []:
        for i in catalogs:
            os.system("rm %s" % (i))
#    create skymaker catalog
    cat = open(fakes + "/sky.cat", "w+")
    config = "/home/andrew/sdi/pipeline/scripts/sky.config"
    data = []
    x = random.randint(0,1526)
    y = random.randint(0,1017)
#    mag = random.uniform(12,20)
    data.append("100" + "   " + str(x) + "   " + str(y) + "   " + str(mag) + "\n")
    for i in data:
        cat.write(i)
    cat.close()
    name = "%s/fakes/%d_%d_%.3f_sky.fits" % (location,x,y,mag)
    with open(config, 'r') as Config:
        data = Config.readlines()
        Config.close()
    data[6] = "IMAGE_NAME" + "        " + name + "\n"
    with open(config, 'w') as Config:
        Config.writelines(data)
        Config.close()
    os.system("sky -c %s %s/fakes/sky.cat" % (config, location))
    images = glob.glob(location + "/data/*_A_.fits")
    fake = glob.glob(location + "/fakes/*.fits")
    comb_name = location + "/data/%d_%d_%.3f_FAKE.fits" % (x,y,mag)
    num = len(images) - 1
    im = random.randint(0,num)
    hdu1 = fits.open(images[im])
    data1 = hdu1[0].data
    Header = hdu1[0].header
    hdu2 = fits.open(fake[0])
    data2 = hdu2[0].data
    comb = data1+data2
    hdu3 = fits.PrimaryHDU(comb, header=Header)
    hdu3.writeto(comb_name)
#    print images[im]
    template = glob.glob(location + "/templates/*.fits")
    fake = glob.glob(location + "/data/*FAKE.fits")
    sub_name = location + "/residuals/FAKE_residual.fits"
    hdu4 = fits.open(template[0])
    data4 = hdu4[0].data
    hdu5 = fits.open(fake[0])
    data5 = hdu5[0].data
    sub = data5 - data4
    hdu6 = fits.PrimaryHDU(sub)
    hdu6.writeto(sub_name)
    combine2(location + "/residuals")
    sex_config = "/home/andrew/sdi/pipeline/scripts/fakesex_config"
    sex_param = "/home/andrew/sdi/pipeline/scripts/fakesex_param"
    fake = glob.glob(location + "/data/*FAKE.fits")
    length = len(location) + 6
    catalog_name = location + "/sources/" + str(fake[0][length:-9]) + ".cat"
    combined_res = glob.glob(location + "/residuals/*COMBINED_residual.fits")
    with open(sex_config, 'r') as Config:
        data = Config.readlines()
        Config.close()
    data[6] = "CATALOG_NAME" + "        " + catalog_name + "\n"
    with open(sex_config, 'w') as Config:
        Config.writelines(data)
        Config.close()
    os.system("sextractor -c %s %s" % (sex_config, combined_res[0]))
    X,Y = sex_cat(location)
    x_high = x+1
    x_low = x-1
    y_high = y+1
    y_low = y-1
    detections = []
    for i in range(len(X)):
        if float(X[i]) >= x_low and float(X[i]) <= x_high and float(Y[i]) >= y_low and float(Y[i]) <= y_high:
            detections.append(i)
    distance_from_core = np.sqrt((754-x)**2+(487-y)**2)
    return detections

#%%
def combine2(location):
    combined = glob.glob(location + "/*COMBINED_residual.fits")
    if combined != []:
        for i in combined:
            os.system("rm %s" % (i))
    data = []
    images = glob.glob(location + "/*.fits") 
    for i in images:
        hdu1 = fits.open(i)
        data1 = hdu1[0].data
        data1 = np.array(data1, dtype="float64")
        data.append(data1)
        Header = hdu1[0].header
        hdu1.close()
    comb = sum(data)
    combined_name = location + "/COMBINED_residual.fits"
    hdu = fits.PrimaryHDU(comb, header=Header)
    hdu.writeto(combined_name)
    
def fake_sub(location):
    fake_res = glob.glob(location + "/residuals/*FAKE_residual.fits")
    if fake_res != []:
        for i in fake_res:
            os.system("rm %s" % (i))
    template = glob.glob(location + "/templates/*.fits")
    fake = glob.glob(location + "/data/*FAKE.fits")
    sub_name = location + "/residuals/FAKE_residual.fits"
    hdu1 = fits.open(template[0])
    data1 = hdu1[0].data
    hdu2 = fits.open(fake[0])
    data2 = hdu2[0].data
    sub = data2 - data1
    hdu3 = fits.PrimaryHDU(sub)
    hdu3.writeto(sub_name)
    combine2(location + "/residuals")
    
def fake_sex(location, target):
    catalogs = glob.glob(location + "/sources/*.cat")
    if catalogs != []:
        for i in catalogs:
            os.system("rm %s" % (i))
    sex_config = "/home/andrew/sdi/pipeline/scripts/fakesex_config"
    sex_param = "/home/andrew/sdi/pipeline/scripts/fakesex_param"
#    fake = glob.glob(location + "/data/*FAKE.fits")
    length = len(location) + 6
    catalog_name = location + "/sources/" + target + ".cat"
    combined_res = glob.glob(location + "/residuals/*COMBINED_residual.fits")
    with open(sex_config, 'r') as Config:
        data = Config.readlines()
        Config.close()
    data[6] = "CATALOG_NAME" + "        " + catalog_name + "\n"
    with open(sex_config, 'w') as Config:
        Config.writelines(data)
        Config.close()
    os.system("sextractor -c %s %s" % (sex_config, combined_res[0]))
    
def sex_cat(location):
    q = 0
    p = 0
    r = 0
    indeces = []
    x = []
    y = []
    class_star = []
    catalog = glob.glob(location + "/sources/*.cat")
    with open(catalog[0], 'r') as cat:
        lines = cat.readlines()
        cat.close()
    for i in lines:
        b = i.split(" ")
        b[:] = [j for j in b if j != '']
        b[:] = b[:3]
        if b[0] != '#':
            r += 1
            x.append(b[0])
            y.append(b[1])
            class_star.append(b[2])
            if float(b[2]) < 0.09:
                indeces.append(q)
            else:
                p += 1
        q += 1
    for j in indeces:
        lines[j] = "#" + "   " + lines[j]
    with open(catalog[0], 'w+') as cat:
        cat.writelines(lines)
        cat.close()
    print "sources = %d | detections = %d" % (r, p)

#P = 0
#D = []
#mag = []
#for i in np.arange(12,22,0.2):
#    for j in range(50):
#        det = fakes("/home/andrew/sdi/simulations/NGC6744/20.0", i)
#        D.append(det)
#        mag.append(i)
#    P += 1
#    print "%.2f%%" % (P/50.0*100)
#    
#for i in range(len(D)):
#    if D[i] == []:
#        D[i] = 0
#    else:
#        D[i] = 1
#    
#detect = []
#for i in range(50):
#    p = 50 + (50*i)
#    q = 0
#    l = i - 1
#    if l >= 0:
#        q = 50 + (50*l)
#    X = D[q:p]
#    detect.append(sum(X)/50.0)

#M = []
#for i in np.arange(12,22,0.2):
#    M.append(i)

#D_M_NGC6744 = []
#for i in detect:
#    D_M_NGC6744.append(i)
    
#M_NGC6744 = []
#for i in M:
#    M_NGC6744.append(i)
    
A, = plt.plot(D_NGC2403, D_D_NGC6744, label='NGC 6744')
B, = plt.plot(D_NGC2403, D_D_NGC3031, label='NGC 3031')
C, = plt.plot(D_NGC2403, D_D_NGC2403, label='NGC 2403')
D, = plt.plot(D_NGC2403, Y, '--',label='core distance threshold')
plt.xlabel('distance to core (arcseconds)')
plt.ylabel('$N_det / N_tot$')
first_legend = plt.legend(handles=[A,B,C])
ax = plt.gca().add_artist(first_legend)
plt.legend(handles=[D], loc=3)
plt.ylim([0,1.4])
plt.show()


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:02:02 2018

@author: andrew
"""

import glob
from astropy.io import fits
import numpy as np

#%%
def subtract2(location):
    length = len(location) + 6
    images = glob.glob(location + "/data/*_A_.fits")
    template = glob.glob(location + "/templates/*.fits")
    hdu1 = fits.open(template[0])
    data1 = hdu1[0].data
    data1 = np.array(data1, dtype="float64")  
#    data1 = data1[:-1]
    for i in images:
        hdu2 = fits.open(i)
        data2 = hdu2[0].data
        data2 = np.array(data2, dtype="float64")
        sub = data2 - data1
        sub_name = location + "/residuals/" + i[length:-5] + "residual.fits"
        hdu = fits.PrimaryHDU(sub, header=hdu2[0].header)
        hdu.writeto(sub_name)
        hdu2.close()
    hdu1.close()
    print("image subtraction successful")


from distutils.core import setup

with open("README", "r") as fh:
    long_description = fh.read()

setup(name='sdi_pipeline',
      version='2.2',
      description='Optical SETI Image Subtraction Pipeline',
      long_description=long_description,
      author='Andrew Stewart',
      url='https://github.com/andrewhstewart/SDI.git',
      author_email='andrew.henry.stewart@emory.edu',
      packages=['sdi_pipeline'],
#      py_modules=['sdi_pipeline.align_astroalign',
#                  'sdi_pipeline.align_iraf',
#                  'sdi_pipeline.align_template',
#                  'sdi_pipeline.align',
#                  'sdi_pipeline.combine_iraf',
#                  'sdi_pipeline.combine_numpy'
#                  'sdi_pipeline.combine_swarp',
#                  'sdi_pipeline.combine',
#                  'sdi_pipeline.get',
#                  'sdi_pipeline.initialize',
#                  'sdi_pipeline.obtain',
#                  'sdi_pipeline.ref_image',
#                  'sdi_pipeline.simulation',
#                  'sdi_pipeline.stats',
#                  'sdi_pipeline.subtract_hotpants',
#                  'sdi_pipeline.subtract_ibi',
#                  'sdi_pipeline.subtract_iraf',
#                  'sdi_pipeline.subtract_numpy',
#                  'sdi_pipeline.subtract',
#                  'sdi_pipeline.pipeline'
#                  'sdi_pipeline.sex',
#                  'sdi_pipeline.sextractor',
#                  'sdi_pipeline.check_saturation'],
      )


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 13:00:46 2018

@author: andrew
"""

import glob
from time import strftime
from time import gmtime
import numpy as np
from astropy.io import fits

#%%
#combine all aligned images in data directory to make a template using the median method and move the temp to the templates dir
def combine_median(location):
    location = location[:-5]
    data = []
    images = glob.glob(location + "/data" + "/*_A_.fits")
    log_loc = location + "/templates/log.txt"
    log_list = open(log_loc, "a+")
    time = strftime("%Y-%m-%d %H:%M:%S", gmtime())
    for i in images:
        hdu1 = fits.open(i)
        data1 = hdu1[0].data
        data1 = np.array(data1, dtype="float64")
        data.append(data1)
        Header = hdu1[0].header
        hdu1.close()
    comb = np.median(data, axis=0)
    combined_name = location + "/templates/median_%d.fits" % (len(images))
    hdu = fits.PrimaryHDU(comb, header=Header)
    hdu.writeto(combined_name)
    log_list.write("template updated at %s UTC | method = median (numpy) | images = %d\n" % (str(time), len(images)))
    log_list.close()
    print("image combination successful!\ntemplate log updated\n")
    return comb


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 12:59:13 2018

@author: andrew
"""

import os
from astropy.io import fits
import glob
from initialize import loc
import numpy as np
import astroalign

#%%
#align images to reference image using astroalign package
def align2(location):
    x = 1
    images = glob.glob(location + "/*_N_.fits")
    ref = glob.glob(location + "/*_ref_A_.fits")
    hdu2 = fits.open(ref[0])
    data2 = hdu2[0].data
    data2 = np.array(data2, dtype="float64")  
    for i in images:
        hdu1 = fits.open(i)
        data1 = hdu1[0].data
        data1 = np.array(data1, dtype="float64")
#        transf, (source_list, target_list) = astroalign.find_transform(data1, data2)
#        aligned = astroalign.apply_transform(transf, data1, data2)
        aligned = astroalign.register(data1, data2)
        aligned_name = i[:-8] + "_A_.fits"
        hdu = fits.PrimaryHDU(aligned, header=hdu1[0].header)
        hdu.writeto(aligned_name)
        hdu1.close()
        os.system("mv %s %s/sdi/archive/data" % (i, loc))
        percent = float(x)/float(len(images)) * 100
        print("%.1f%% aligned..." % (percent))
        x += 1
    hdu2.close()
    


from os.path import expanduser
import os

loc = expanduser("~")

#%%
#run on a new computer to create the sdi file system and use the pipeline
def initialize(loc):
    if os.path.exists(loc + "/sdi") == False:
        os.system("mkdir %s/sdi" % (loc))
        os.system("mkdir %s/sdi/targets" % (loc))
        os.system("mkdir %s/sdi/temp" % (loc))
        os.system("mkdir %s/sdi/sources" % (loc))
        os.system("mkdir %s/sdi/archive" % (loc))
        os.system("mkdir %s/sdi/pipeline" % (loc))
        os.system("mkdir %s/sdi/observations" % (loc))
        os.system("mkdir %s/sdi/simulations" % (loc))
        os.system("mkdir %s/sdi/archive/data" % (loc))
        os.system("mkdir %s/sdi/archive/templates" % (loc))
        os.system("mkdir %s/sdi/archive/residuals" % (loc))
        os.system("mkdir %s/sdi/scripts" % (loc))
        print("sdi file system created in %s\n" % (loc))
    else:
        print("SDI architecure already exists on this computer")
    
#%%
#creates a data, template, and residual directory
def create(location):
    dirs = ["data", "templates", "residuals", "sources", "psf"]
    for d in dirs: os.system("mkdir %s/%s" % (location, d))
    print("data, templates, residuals, sources, and psf directories created in %s\n" % (location))

#%%
#if this architecture does not exist, create it
if __name__ == '__main__':
    alert = input("Create SDI directories in %s? (y/n)\n" % (loc))
    if alert == 'y':
        initialize(loc)
    elif alert == 'n':
        print("Change loc variable in initialize.py to desired SDI directory path, then run script again")
    else:
        print("Error: unknown input")